@@ -1,21 +1,31 @@
-##	gtkgui_helpers.py
+# -*- coding:utf-8 -*-
+## src/gtkgui_helpers.py
 ##
-## Copyright (C) 2003-2006 Yann Le Boulanger <asterix@lagaule.org>
-## Copyright (C) 2004-2005 Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
-## Copyright (C) 2005 Dimitur Kirov <dkirov@gmail.com>
-## Copyright (C) 2005 Travis Shirk <travis@pobox.com>
-## Copyright (C) 2005 Norman Rasmussen <norman@rasmussen.co.za>
+## Copyright (C) 2003-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2005-2006 Dimitur Kirov <dkirov AT gmail.com>
+## Copyright (C) 2005-2007 Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2006 Travis Shirk <travis AT pobox.com>
+## Copyright (C) 2006-2007 Junglecow J <junglecow AT gmail.com>
+## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
+## Copyright (C) 2007 James Newton <redshodan AT gmail.com>
+##                    Julien Pivotto <roidelapluie AT gmail.com>
+## Copyright (C) 2007-2008 Stephan Erb <steve-e AT h3c.de>
+## Copyright (C) 2008 Jonathan Schleifer <js-gajim AT webkeks.org>
 ##
-## This program is free software; you can redistribute it and/or modify
+## This file is part of Gajim.
+##
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
 import xml.sax.saxutils
 import gtk
@@ -28,6 +38,9 @@ import sys
 import vcard
 import dialogs
 
+import logging
+log = logging.getLogger('gajim.gtkgui_helpers')
+
 
 HAS_PYWIN32 = True
 if os.name == 'nt':
@@ -118,7 +131,7 @@ def get_theme_font_for_option(theme, option):
 	
 def get_default_font():
 	'''Get the desktop setting for application font
-	first check for GNOME, then XFCE and last KDE
+	first check for GNOME, then Xfce and last KDE
 	it returns None on failure or else a string 'Font Size' '''
 	
 	try:
@@ -128,7 +141,7 @@ def get_default_font():
 
 		return client.get_string('/desktop/gnome/interface/font_name'
 			).decode('utf-8')
-	except:
+	except Exception:
 		pass
 
 	# try to get xfce default font
@@ -144,17 +157,17 @@ def get_default_font():
 	
 	if os.path.exists(xfce_config_file):
 		try:
-			for line in file(xfce_config_file):
+			for line in open(xfce_config_file):
 				if line.find('name="Gtk/FontName"') != -1:
 					start = line.find('value="') + 7
 					return line[start:line.find('"', start)].decode('utf-8')
-		except:
+		except Exception:
 			#we talk about file
 			print >> sys.stderr, _('Error: cannot open %s for reading') % xfce_config_file
 	
 	elif os.path.exists(kde_config_file):
 		try:
-			for line in file(kde_config_file):
+			for line in open(kde_config_file):
 				if line.find('font=') == 0: # font=Verdana,9,other_numbers
 					start = 5 # 5 is len('font=')
 					line = line[start:]
@@ -163,25 +176,12 @@ def get_default_font():
 					font_size = values[1]
 					font_string = '%s %s' % (font_name, font_size) # Verdana 9
 					return font_string.decode('utf-8')
-		except:
+		except Exception:
 			#we talk about file
 			print >> sys.stderr, _('Error: cannot open %s for reading') % kde_config_file
 	
 	return None
 	
-def escape_for_pango_markup(string):
-	# escapes < > & ' "
-	# for pango markup not to break
-	if string is None:
-		return
-	if gtk.pygtk_version >= (2, 8, 0) and gtk.gtk_version >= (2, 8, 0):
-		escaped_str = gobject.markup_escape_text(string)
-	else:
-		escaped_str = xml.sax.saxutils.escape(string, {"'": '&apos;',
-			'"': '&quot;'})
-	
-	return escaped_str
-
 def autodetect_browser_mailer():
 	# recognize the environment and set appropriate browser/mailer
 	if user_runs_gnome():
@@ -190,6 +190,8 @@ def autodetect_browser_mailer():
 		gajim.config.set('openwith', 'kfmclient exec')
 	elif user_runs_xfce():
 		gajim.config.set('openwith', 'exo-open')
+	elif user_runs_osx():
+		gajim.config.set('openwith', 'open')
 	else:
 		gajim.config.set('openwith', 'custom')
 
@@ -205,6 +207,9 @@ def user_runs_xfce():
 		return True
 	return False
 
+def user_runs_osx():
+	return sys.platform == 'darwin'
+
 def get_running_processes():
 	'''returns running processes or None (if not /proc exists)'''
 	if os.path.isdir('/proc'):
@@ -246,6 +251,11 @@ def move_window(window, x, y):
 		x = 0
 	if y < 0:
 		y = 0
+	w, h = window.get_size()
+	if x + w > screen_w:
+		x = screen_w - w
+	if y + h > screen_h:
+		y = screen_h - h
 	window.move(x, y)
 
 def resize_window(window, w, h):
@@ -258,34 +268,87 @@ def resize_window(window, w, h):
 		h = screen_h
 	window.resize(abs(w), abs(h))
 
-class TagInfoHandler(xml.sax.ContentHandler):
-	def __init__(self, tagname1, tagname2):
+class HashDigest:
+	def __init__(self, algo, digest):
+		self.algo = self.cleanID(algo)
+		self.digest = self.cleanID(digest)
+
+	def cleanID(self, id_):
+		id_ = id_.strip().lower()
+		for strip in (' :.-_'): id_ = id_.replace(strip, '')
+		return id_
+
+	def __eq__(self, other):
+		sa, sd = self.algo, self.digest
+		if isinstance(other, self.__class__):
+			oa, od = other.algo, other.digest
+		elif isinstance(other, basestring):
+			sa, oa, od = None, None, self.cleanID(other)
+		elif isinstance(other, tuple) and len(other) == 2:
+			oa, od = self.cleanID(other[0]), self.cleanID(other[1])
+		else:
+			return False
+
+		return sa == oa and sd == od
+
+	def __ne__(self, other):
+		return not self == other
+
+	def __hash__(self):
+		return self.algo ^ self.digest
+
+	def __str__(self):
+		prettydigest = ''
+		for i in xrange(0, len(self.digest), 2):
+			prettydigest += self.digest[i:i + 2] + ':'
+		return prettydigest[:-1]
+
+	def __repr__(self):
+		return "%s(%s, %s)" % (self.__class__, repr(self.algo), repr(str(self)))
+
+class ServersXMLHandler(xml.sax.ContentHandler):
+	def __init__(self):
 		xml.sax.ContentHandler.__init__(self)
-		self.tagname1 = tagname1
-		self.tagname2 = tagname2
 		self.servers = []
 
 	def startElement(self, name, attributes):
-		if name == self.tagname1:
+		if name == 'item':
+			# we will get the port next time so we just set it 0 here
+			sitem = [None, 0, {}]
+			sitem[2]['digest'] = {}
+			sitem[2]['hidden'] = False
 			for attribute in attributes.getNames():
 				if attribute == 'jid':
 					jid = attributes.getValue(attribute)
-					# we will get the port next time so we just set it 0 here
-					self.servers.append([jid, 0])
-		elif name == self.tagname2:
+					sitem[0] = jid
+				elif attribute == 'hidden':
+					hidden = attributes.getValue(attribute)
+					if hidden.lower() in ('1', 'y', 'yes', 't', 'true', 'on'):
+						sitem[2]['hidden'] = True
+			self.servers.append(sitem)
+		elif name == 'active':
 			for attribute in attributes.getNames():
 				if attribute == 'port':
 					port = attributes.getValue(attribute)
 					# we received the jid last time, so we now assign the port
 					# number to the last jid in the list
 					self.servers[-1][1] = port
+		elif name == 'digest':
+			algo, digest = None, None
+			for attribute in attributes.getNames():
+				if attribute == 'algo':
+					algo = attributes.getValue(attribute)
+				elif attribute == 'value':
+					digest = attributes.getValue(attribute)
+			hd = HashDigest(algo, digest)
+			self.servers[-1][2]['digest'][hd.algo] = hd
 
 	def endElement(self, name):
 		pass
 
 def parse_server_xml(path_to_file):
 	try:
-		handler = TagInfoHandler('item', 'active')
+		handler = ServersXMLHandler()
 		xml.sax.parse(path_to_file, handler)
 		return handler.servers
 	# handle exception if unable to open file
@@ -298,8 +361,7 @@ def parse_server_xml(path_to_file):
 def set_unset_urgency_hint(window, unread_messages_no):
 	'''sets/unsets urgency hint in window argument
 	depending if we have unread messages or not'''
-	if gtk.gtk_version >= (2, 8, 0) and gtk.pygtk_version >= (2, 8, 0) and \
-		gajim.config.get('use_urgency_hint'):
+	if gajim.config.get('use_urgency_hint'):
 		if unread_messages_no > 0:
 			window.props.urgency_hint = True
 		else:
@@ -354,7 +416,12 @@ def get_pixbuf_from_data(file_data, want_type = False):
 		pixbufloader.close()
 		pixbuf = pixbufloader.get_pixbuf()
 	except gobject.GError: # 'unknown image format'
+		pixbufloader.close()
 		pixbuf = None
+		if want_type:
+			return None, None
+		else:
+			return None
 
 	if want_type:
 		typ = pixbufloader.get_format()['name']
@@ -450,8 +517,8 @@ def _get_fade_color(treeview, selected, focused):
 	p = 0.3 # background
 	q = 0.7 # foreground # p + q should do 1.0
 	return gtk.gdk.Color(int(bg.red*p + fg.red*q),
-			      int(bg.green*p + fg.green*q),
-			      int(bg.blue*p + fg.blue*q))
+					int(bg.green*p + fg.green*q),
+					int(bg.blue*p + fg.blue*q))
 
 def get_scaled_pixbuf(pixbuf, kind):
 	'''returns scaled pixbuf, keeping ratio etc or None
@@ -481,7 +548,7 @@ def get_scaled_pixbuf(pixbuf, kind):
 	scaled_buf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_HYPER)
 	return scaled_buf
 
-def get_avatar_pixbuf_from_cache(fjid, is_fake_jid = False):
+def get_avatar_pixbuf_from_cache(fjid, is_fake_jid = False, use_local = True):
 	'''checks if jid has cached avatar and if that avatar is valid image
 	(can be shown)
 	returns None if there is no image in vcard
@@ -498,8 +565,21 @@ def get_avatar_pixbuf_from_cache(fjid, is_fake_jid = False):
 	if is_fake_jid:
 		puny_nick = helpers.sanitize_filename(nick)
 		path = os.path.join(gajim.VCARD_PATH, puny_jid, puny_nick)
+		local_avatar_basepath = os.path.join(gajim.AVATAR_PATH, puny_jid,
+			puny_nick) + '_local'
 	else:
 		path = os.path.join(gajim.VCARD_PATH, puny_jid)
+		local_avatar_basepath = os.path.join(gajim.AVATAR_PATH, puny_jid) + \
+			'_local'
+	if use_local:
+		for extension in ('.png', '.jpeg'):
+			local_avatar_path = local_avatar_basepath + extension
+			if os.path.isfile(local_avatar_path):
+				avatar_file = open(local_avatar_path, 'rb')
+				avatar_data = avatar_file.read()
+				avatar_file.close()
+				return get_pixbuf_from_data(avatar_data)
+
 	if not os.path.isfile(path):
 		return 'ask'
 
@@ -507,7 +587,7 @@ def get_avatar_pixbuf_from_cache(fjid, is_fake_jid = False):
 		is_fake_jid)
 	if not vcard_dict: # This can happen if cached vcard is too old
 		return 'ask'
-	if not vcard_dict.has_key('PHOTO'):
+	if 'PHOTO' not in vcard_dict:
 		return None
 	pixbuf = vcard.get_avatar_pixbuf_encoded_mime(vcard_dict['PHOTO'])[0]
 	return pixbuf
@@ -524,9 +604,8 @@ def make_python_month_gtk_month(month):
 def make_color_string(color):
 	'''create #aabbcc color string from gtk color'''
 	col = '#'
-	for i in (color.red, color.green, color.blue):
-		# GTK sometime return a value > 256
-		h = hex(i%256)[2:4]
+	for i in ('red', 'green', 'blue'):
+		h = hex(getattr(color, i) / (16*16)).split('x')[1]
 		if len(h) == 1:
 			h = '0' + h
 		col += h
@@ -540,12 +619,21 @@ def make_pixbuf_grayscale(pixbuf):
 def get_path_to_generic_or_avatar(generic, jid = None, suffix = None):
 	'''Chooses between avatar image and default image.
 	Returns full path to the avatar image if it exists,
-	otherwise returns full path to the image.'''
+	otherwise returns full path to the image.
+	generic must be with extension and suffix without'''
 	if jid:
+		# we want an avatar
 		puny_jid = helpers.sanitize_filename(jid)
 		path_to_file = os.path.join(gajim.AVATAR_PATH, puny_jid) + suffix
-		if os.path.exists(path_to_file):
-			return path_to_file
+		path_to_local_file = path_to_file + '_local'
+		for extension in ('.png', '.jpeg'):
+			path_to_local_file_full = path_to_local_file + extension
+			if os.path.exists(path_to_local_file_full):
+				return path_to_local_file_full 
+		for extension in ('.png', '.jpeg'):
+			path_to_file_full = path_to_file + extension
+			if os.path.exists(path_to_file_full):
+				return path_to_file_full
 	return os.path.abspath(generic)
 
 def decode_filechooser_file_paths(file_paths):
@@ -562,10 +650,10 @@ def decode_filechooser_file_paths(file_paths):
 		for file_path in file_paths:
 			try:
 				file_path = file_path.decode(sys.getfilesystemencoding())
-			except:
+			except Exception:
 				try:
 					file_path = file_path.decode('utf-8')
-				except:
+				except Exception:
 					pass
 			file_paths_list.append(file_path)
 	
@@ -573,54 +661,34 @@ def decode_filechooser_file_paths(file_paths):
 
 def possibly_set_gajim_as_xmpp_handler():
 	'''registers (by default only the first time) xmmp: to Gajim.'''
-	try:
-		import gconf
-		# in try because daemon may not be there
-		client = gconf.client_get_default()
-	except:
-		return
-	
-	
 	path_to_dot_kde = os.path.expanduser('~/.kde')
 	if os.path.exists(path_to_dot_kde):
 		path_to_kde_file = os.path.join(path_to_dot_kde, 
 			'share/services/xmpp.protocol')
-		if not os.path.exists(path_to_kde_file):
-			path_to_kde_file = None
 	else:
 		path_to_kde_file = None
-	
-	if gajim.config.get('set_xmpp://_handler_everytime'):
-			# it's false by default
-			we_set = True
-	elif client.get_string('/desktop/gnome/url-handlers/xmpp/command') is None:
-		# only the first time (GNOME/GCONF)
-		we_set = True
-	elif path_to_kde_file is not None and not os.path.exists(path_to_kde_file):
-		# only the first time (KDE)
-		we_set = True
-	else:
-		we_set = False
-		
-	if not we_set:
-		return
-	
-	path_to_gajim_script, typ = get_abspath_for_script('gajim-remote', True)
-	if path_to_gajim_script:
-		if typ == 'svn':
-			command = path_to_gajim_script + ' open_chat %s'
-		else: # 'installed'
-			command = 'gajim-remote open_chat %s'
-		
-		# setting for GNOME/Gconf
-		client.set_bool('/desktop/gnome/url-handlers/xmpp/enabled', True)
-		client.set_string('/desktop/gnome/url-handlers/xmpp/command', command)
-		client.set_bool('/desktop/gnome/url-handlers/xmpp/needs_terminal', False)
-		
-		# setting for KDE
-		if path_to_kde_file is not None: # user has run kde at least once
-			f = open(path_to_kde_file, 'w')
-			f.write('''\
+
+	def set_gajim_as_xmpp_handler(is_checked=None):
+		if is_checked is not None:
+			# come from confirmation dialog
+			gajim.config.set('check_if_gajim_is_default', is_checked)
+		path_to_gajim_script, typ = get_abspath_for_script('gajim-remote', True)
+		if path_to_gajim_script:
+			if typ == 'svn':
+				command = path_to_gajim_script + ' handle_uri %s'
+			else: # 'installed'
+				command = 'gajim-remote handle_uri %s'
+
+			# setting for GNOME/Gconf
+			client.set_bool('/desktop/gnome/url-handlers/xmpp/enabled', True)
+			client.set_string('/desktop/gnome/url-handlers/xmpp/command', command)
+			client.set_bool('/desktop/gnome/url-handlers/xmpp/needs_terminal', False)
+
+			# setting for KDE
+			if path_to_kde_file is not None: # user has run kde at least once
+				try:
+					f = open(path_to_kde_file, 'a')
+					f.write('''\
 [Protocol]
 exec=%s "%%u"
 protocol=xmpp
@@ -635,7 +703,43 @@ deleting=false
 icon=gajim
 Description=xmpp
 ''' % command)
-			f.close()
+					f.close()
+				except IOError:
+					log.debug("I/O Error writing settings to %s", repr(path_to_kde_file), exc_info=True)
+		else: # no gajim remote, stop ask user everytime
+			gajim.config.set('check_if_gajim_is_default', False)
+
+	try:
+		import gconf
+		# in try because daemon may not be there
+		client = gconf.client_get_default()
+	except Exception:
+		return
+
+	old_command = client.get_string('/desktop/gnome/url-handlers/xmpp/command')
+	if not old_command or old_command.endswith(' open_chat %s'):
+		# first time (GNOME/GCONF) or old Gajim version
+		we_set = True
+	elif path_to_kde_file is not None and not os.path.exists(path_to_kde_file):
+		# only the first time (KDE)
+		we_set = True
+	else:
+		we_set = False
+
+	if we_set:
+		set_gajim_as_xmpp_handler()
+	elif old_command and not old_command.endswith(' handle_uri %s'):
+		# xmpp: is currently handled by another program, so ask the user
+		pritext = _('Gajim is not the default Jabber client')
+		sectext = _('Would you like to make Gajim the default Jabber client?')
+		checktext = _('Always check to see if Gajim is the default Jabber client '
+			'on startup')
+		def on_cancel(checked):
+			gajim.config.set('check_if_gajim_is_default', checked)
+		dlg = dialogs.ConfirmationDialogCheck(pritext, sectext, checktext,
+			set_gajim_as_xmpp_handler, on_cancel)
+		if gajim.config.get('check_if_gajim_is_default'):
+			dlg.checkbutton.set_active(True)
 
 def escape_underscore(s):
 	'''Escape underlines to prevent them from being interpreted
@@ -668,45 +772,15 @@ def destroy_widget(widget):
 
 def on_avatar_save_as_menuitem_activate(widget, jid, account,
 default_name = ''):
-	def on_ok(widget):
-		def on_ok2(widget, file_path, pixbuf):
-			pixbuf.save(file_path, 'jpeg')
-			dialog2.destroy()
-			dialog.destroy()
-
-		file_path = dialog.get_filename()
-		file_path = decode_filechooser_file_paths((file_path,))[0]
-		if os.path.exists(file_path):
-			# check if we have write permissions
-			if not os.access(file_path, os.W_OK):
-				file_name = os.path.basename(file_path)
-				dialogs.ErrorDialog(_('Cannot overwrite existing file "%s"' % 
-					file_name),
-				_('A file with this name already exists and you do not have '
-				'permission to overwrite it.'))
-				return
-			dialog2 = dialogs.FTOverwriteConfirmationDialog(
-				_('This file already exists'), _('What do you want to do?'),
-				False)
-			dialog2.set_transient_for(dialog)
-			dialog2.set_destroy_with_parent(True)
-			response = dialog2.get_response()
-			if response < 0:
-				return
-		else:
-			dirname = os.path.dirname(file_path)
-			if not os.access(dirname, os.W_OK):
-				dialogs.ErrorDialog(_('Directory "%s" is not writable') % \
-				dirname, _('You do not have permission to create files in this'
-				' directory.'))
-				return
-
+	def on_continue(response, file_path):
+		if response < 0:
+			return
 		# Get pixbuf
 		pixbuf = None
 		is_fake = False
 		if account and gajim.contacts.is_pm_from_jid(account, jid):
 			is_fake = True
-		pixbuf = get_avatar_pixbuf_from_cache(jid, is_fake)
+		pixbuf = get_avatar_pixbuf_from_cache(jid, is_fake, False)
 		ext = file_path.split('.')[-1]
 		type_ = ''
 		if not ext:
@@ -725,24 +799,54 @@ default_name = ''):
 			if os.path.exists(file_path):
 				os.remove(file_path)
 			new_file_path = '.'.join(file_path.split('.')[:-1]) + '.jpeg'
-			dialog2 = dialogs.ConfirmationDialog(_('Extension not supported'),
+			def on_ok(file_path, pixbuf):
+				pixbuf.save(file_path, 'jpeg')
+			dialogs.ConfirmationDialog(_('Extension not supported'),
 				_('Image cannot be saved in %(type)s format. Save as %(new_filename)s?') % {'type': type_, 'new_filename': new_file_path},
-				on_response_ok = (on_ok2, new_file_path, pixbuf))
+				on_response_ok = (on_ok, new_file_path, pixbuf))
 		else:
 			dialog.destroy()
 
+	def on_ok(widget):
+		def on_ok2(file_path, pixbuf):
+			pixbuf.save(file_path, 'jpeg')
+			dialog.destroy()
+
+		file_path = dialog.get_filename()
+		file_path = decode_filechooser_file_paths((file_path,))[0]
+		if os.path.exists(file_path):
+			# check if we have write permissions
+			if not os.access(file_path, os.W_OK):
+				file_name = os.path.basename(file_path)
+				dialogs.ErrorDialog(_('Cannot overwrite existing file "%s"' % 
+					file_name),
+				_('A file with this name already exists and you do not have '
+				'permission to overwrite it.'))
+				return
+			dialog2 = dialogs.FTOverwriteConfirmationDialog(
+				_('This file already exists'), _('What do you want to do?'),
+				propose_resume=False, on_response=(on_continue, file_path))
+			dialog2.set_transient_for(dialog)
+			dialog2.set_destroy_with_parent(True)
+		else:
+			dirname = os.path.dirname(file_path)
+			if not os.access(dirname, os.W_OK):
+				dialogs.ErrorDialog(_('Directory "%s" is not writable') % \
+				dirname, _('You do not have permission to create files in this'
+				' directory.'))
+				return
+
+		on_continue(0, file_path)
+
 	def on_cancel(widget):
 		dialog.destroy()
 
-	dialog = dialogs.FileChooserDialog(
-		title_text = _('Save Image as...'), 
-		action = gtk.FILE_CHOOSER_ACTION_SAVE, 
-		buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, 
-		gtk.STOCK_SAVE, gtk.RESPONSE_OK),
-		default_response = gtk.RESPONSE_OK,
-		current_folder = gajim.config.get('last_save_dir'),
-		on_response_ok = on_ok,
-		on_response_cancel = on_cancel)
+	dialog = dialogs.FileChooserDialog(title_text=_('Save Image as...'), 
+		action=gtk.FILE_CHOOSER_ACTION_SAVE, buttons=(gtk.STOCK_CANCEL,
+		gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_OK),
+		default_response=gtk.RESPONSE_OK,
+		current_folder=gajim.config.get('last_save_dir'), on_response_ok=on_ok,
+		on_response_cancel=on_cancel)
 
 	dialog.set_current_name(default_name)
 	dialog.connect('delete-event', lambda widget, event:
@@ -750,3 +854,136 @@ default_name = ''):
 
 def on_bm_header_changed_state(widget, event):
 	widget.set_state(gtk.STATE_NORMAL) #do not allow selected_state
+
+def create_combobox(value_list, selected_value = None):
+	'''Value_list is [(label1, value1), ]'''
+	liststore = gtk.ListStore(str, str)
+	combobox = gtk.ComboBox(liststore)
+	cell = gtk.CellRendererText()
+	combobox.pack_start(cell, True)
+	combobox.add_attribute(cell, 'text', 0)
+	i = -1
+	for value in value_list:
+		liststore.append(value)
+		if selected_value == value[1]:
+			i = value_list.index(value)
+	if i > -1:
+		combobox.set_active(i)
+	combobox.show_all()
+	return combobox
+
+def load_iconset(path, pixbuf2 = None, transport = False):
+	'''load full iconset from the given path, and add
+	pixbuf2 on top left of each static images'''
+	path += '/'
+	if transport:
+		list = ('online', 'chat', 'away', 'xa', 'dnd', 'offline',
+			'not in roster')
+	else:
+		list = ('connecting', 'online', 'chat', 'away', 'xa', 'dnd',
+			'invisible', 'offline', 'error', 'requested', 'event', 'opened',
+			'closed', 'not in roster', 'muc_active', 'muc_inactive')
+		if pixbuf2:
+			list = ('connecting', 'online', 'chat', 'away', 'xa', 'dnd',
+				'offline', 'error', 'requested', 'event', 'not in roster')
+	return _load_icon_list(list, path, pixbuf2)
+
+def load_icon(icon_name):
+	'''load an icon from the iconset in 16x16'''
+	iconset = gajim.config.get('iconset')
+	path = os.path.join(helpers.get_iconset_path(iconset), '16x16', '')
+	icon_list = _load_icon_list([icon_name], path)
+	return icon_list[icon_name]
+
+def load_mood_icon(icon_name):
+	'''load an icon from the mood iconset in 16x16'''
+	iconset = gajim.config.get('mood_iconset')
+	path = os.path.join(helpers.get_mood_iconset_path(iconset), '')
+	icon_list = _load_icon_list([icon_name], path)
+	return icon_list[icon_name]
+
+def load_activity_icon(category, activity = None):
+	'''load an icon from the activity iconset in 16x16'''
+	iconset = gajim.config.get('activity_iconset')
+	path = os.path.join(helpers.get_activity_iconset_path(iconset),
+		category, '')
+	if activity is None:
+		activity = 'category'
+	icon_list = _load_icon_list([activity], path)
+	return icon_list[activity]
+
+def load_icons_meta():
+	'''load and return  - AND + small icons to put on top left of an icon
+	for meta contacts.'''
+	iconset = gajim.config.get('iconset')
+	path = os.path.join(helpers.get_iconset_path(iconset), '16x16')
+	# try to find opened_meta.png file, else opened.png else nopixbuf merge
+	path_opened = os.path.join(path, 'opened_meta.png')
+	if not os.path.isfile(path_opened):
+		path_opened = os.path.join(path, 'opened.png')
+	if os.path.isfile(path_opened):
+		pixo = gtk.gdk.pixbuf_new_from_file(path_opened)
+	else:
+		pixo = None
+	# Same thing for closed
+	path_closed = os.path.join(path, 'opened_meta.png')
+	if not os.path.isfile(path_closed):
+		path_closed = os.path.join(path, 'closed.png')
+	if os.path.isfile(path_closed):
+		pixc = gtk.gdk.pixbuf_new_from_file(path_closed)
+	else:
+		pixc = None
+	return pixo, pixc
+
+def _load_icon_list(icons_list, path, pixbuf2 = None):
+	'''load icons in icons_list from the given path,
+	and add pixbuf2 on top left of each static images'''
+	imgs = {}
+	for icon in icons_list:
+		# try to open a pixfile with the correct method
+		icon_file = icon.replace(' ', '_')
+		files = []
+		files.append(path + icon_file + '.gif')
+		files.append(path + icon_file + '.png')
+		image = gtk.Image()
+		image.show()
+		imgs[icon] = image
+		for file in files: # loop seeking for either gif or png
+			if os.path.exists(file):
+				image.set_from_file(file)
+				if pixbuf2 and image.get_storage_type() == gtk.IMAGE_PIXBUF:
+					# add pixbuf2 on top-left corner of image
+					pixbuf1 = image.get_pixbuf()
+					pixbuf2.composite(pixbuf1, 0, 0,
+						pixbuf2.get_property('width'),
+						pixbuf2.get_property('height'), 0, 0, 1.0, 1.0,
+						gtk.gdk.INTERP_NEAREST, 255)
+					image.set_from_pixbuf(pixbuf1)
+				break
+	return imgs
+
+def make_jabber_state_images():
+	'''initialise jabber_state_images dict'''
+	iconset = gajim.config.get('iconset')
+	if iconset:
+		path = os.path.join(helpers.get_iconset_path(iconset), '16x16')
+		if not os.path.exists(path):
+			iconset = gajim.config.DEFAULT_ICONSET
+	else:
+		iconset = gajim.config.DEFAULT_ICONSET
+
+	path = os.path.join(helpers.get_iconset_path(iconset), '32x32')
+	gajim.interface.jabber_state_images['32'] = load_iconset(path)
+
+	path = os.path.join(helpers.get_iconset_path(iconset), '16x16')
+	gajim.interface.jabber_state_images['16'] = load_iconset(path)
+
+	pixo, pixc = load_icons_meta()
+	gajim.interface.jabber_state_images['opened'] = load_iconset(path, pixo)
+	gajim.interface.jabber_state_images['closed'] = load_iconset(path, pixc)
+
+def reload_jabber_state_images():
+	make_jabber_state_images()
+	gajim.interface.roster.update_jabber_state_images()
+
+# vim: se ts=3:
