@@ -1,18 +1,32 @@
-##	tooltips.py
+# -*- coding: utf-8 -*-
+## src/tooltips.py
 ##
-## Copyright (C) 2005-2006 Dimitur Kirov <dkirov@gmail.com>
-## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
-## Copyright (C) 2005-2006 Yann Le Boulanger <asterix@lagaule.org>
+## Copyright (C) 2005 Alex Mauer <hawke AT hawkesnest.net>
+##                    Stéphan Kochen <stephan AT kochen.nl>
+## Copyright (C) 2005-2006 Dimitur Kirov <dkirov AT gmail.com>
+## Copyright (C) 2005-2007 Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2005-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2006 Travis Shirk <travis AT pobox.com>
+##                    Stefan Bethge <stefan AT lanpartei.de>
+## Copyright (C) 2006-2007 Jean-Marie Traissard <jim AT lapin.org>
+## Copyright (C) 2007 Julien Pivotto <roidelapluie AT gmail.com>
+## Copyright (C) 2007-2008 Stephan Erb <steve-e AT h3c.de>
+## Copyright (C) 2008 Jonathan Schleifer <js-gajim AT webkeks.org>
 ##
-## This program is free software; you can redistribute it and/or modify
+## This file is part of Gajim.
+##
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
 import gtk
 import gobject
@@ -24,7 +38,7 @@ import gtkgui_helpers
 
 from common import gajim
 from common import helpers
-from common import i18n
+from common.pep import MOODS, ACTIVITIES
 
 class BaseTooltip:
 	''' Base Tooltip class;
@@ -90,7 +104,7 @@ class BaseTooltip:
 		half_width = requisition.width / 2 + 1
 		if self.preferred_position[0] < half_width: 
 			self.preferred_position[0] = 0
-		elif self.preferred_position[0]  + requisition.width > \
+		elif self.preferred_position[0] + requisition.width > \
 			self.screen.get_width() + half_width:
 			self.preferred_position[0] = self.screen.get_width() - \
 				requisition.width
@@ -100,7 +114,7 @@ class BaseTooltip:
 		if self.preferred_position[1] + requisition.height > \
 			self.screen.get_height():
 			# flip tooltip up
-			self.preferred_position[1] -= requisition.height  + \
+			self.preferred_position[1] -= requisition.height + \
 				self.widget_height + 8
 		if self.preferred_position[1] < 0:
 			self.preferred_position[1] = 0
@@ -162,13 +176,15 @@ class StatusTable:
 		self.table = gtk.Table(4, 1)
 		self.table.set_property('column-spacing', 2)
 	
-	def add_text_row(self, text):
+	def add_text_row(self, text, col_inc = 0):
+		self.current_row += 1
 		self.text_label = gtk.Label()
 		self.text_label.set_line_wrap(True)
 		self.text_label.set_alignment(0, 0)
 		self.text_label.set_selectable(False)
 		self.text_label.set_markup(text)
-		self.table.attach(self.text_label, 1, 4, 1, 2)
+		self.table.attach(self.text_label, 1 + col_inc, 4, self.current_row,
+			self.current_row + 1)
 		
 	def get_status_info(self, resource, priority, show, status):
 		str_status = resource + ' (' + unicode(priority) + ')'
@@ -180,12 +196,13 @@ class StatusTable:
 					status = unicode(status, encoding='utf-8')
 				# reduce to 100 chars, 1 line
 				status = helpers.reduce_chars_newlines(status, 100, 1)
-				str_status = gtkgui_helpers.escape_for_pango_markup(str_status)
-				status = gtkgui_helpers.escape_for_pango_markup(status)
+				str_status = gobject.markup_escape_text(str_status)
+				status = gobject.markup_escape_text(status)
 				str_status += ' - <i>' + status + '</i>'
 		return str_status
 	
-	def add_status_row(self, file_path, show, str_status, status_time = None, show_lock = False):
+	def add_status_row(self, file_path, show, str_status, status_time=None,
+	show_lock=False, indent=True):
 		''' appends a new row with status icon to the table '''
 		self.current_row += 1
 		state_file = show.replace(' ', '_')
@@ -200,8 +217,9 @@ class StatusTable:
 				break
 		spacer = gtk.Label(self.spacer_label)
 		image.set_alignment(1, 0.5)
-		self.table.attach(spacer, 1, 2, self.current_row, 
-			self.current_row + 1, 0, 0, 0, 0)
+		if indent:
+			self.table.attach(spacer, 1, 2, self.current_row, 
+				self.current_row + 1, 0, 0, 0, 0)
 		self.table.attach(image, 2, 3, self.current_row, 
 			self.current_row + 1, gtk.FILL, gtk.FILL, 2, 0)
 		status_label = gtk.Label()
@@ -227,7 +245,7 @@ class NotificationAreaTooltip(BaseTooltip, StatusTable):
 		iconset = gajim.config.get('iconset')
 		if not iconset:
 			iconset = 'dcraven'
-		file_path = os.path.join(gajim.DATA_DIR, 'iconsets', iconset, '16x16')
+		file_path = os.path.join(helpers.get_iconset_path(iconset), '16x16')
 		for acct in accounts:
 			message = acct['message']
 			# before reducing the chars we should assure we send unicode, else 
@@ -235,35 +253,33 @@ class NotificationAreaTooltip(BaseTooltip, StatusTable):
 			if isinstance(message, str):
 				message = unicode(message, encoding = 'utf-8')
 			message = helpers.reduce_chars_newlines(message, 100, 1)
-			message = gtkgui_helpers.escape_for_pango_markup(message)
-			if gajim.con_types.has_key(acct['name']) and \
+			message = gobject.markup_escape_text(message)
+			if acct['name'] in gajim.con_types and \
 				gajim.con_types[acct['name']] in ('tls', 'ssl'):
 				show_lock = True
 			else:
 				show_lock = False
 			if message:
 				self.add_status_row(file_path, acct['show'], 
-					gtkgui_helpers.escape_for_pango_markup(acct['name']) + \
-					' - ' + message, show_lock=show_lock)
+					gobject.markup_escape_text(acct['name']) + \
+					' - ' + message, show_lock=show_lock, indent=False)
 			else:
 				self.add_status_row(file_path, acct['show'], 
-					gtkgui_helpers.escape_for_pango_markup(acct['name']) 
-					, show_lock=show_lock)
+					gobject.markup_escape_text(acct['name']) 
+					, show_lock=show_lock, indent=False)
+			for line in acct['event_lines']:
+				self.add_text_row('  ' + line, 1)
 
 	def populate(self, data):
 		self.create_window()
 		self.create_table()
-		accounts = helpers.get_accounts_info()
-		if len(accounts) > 1:
-			self.table.resize(2, 1)
-			self.fill_table_with_accounts(accounts)
+
+		accounts = helpers.get_notification_icon_tooltip_dict()
+		self.table.resize(2, 1)
+		self.fill_table_with_accounts(accounts)
 		self.hbox = gtk.HBox()
 		self.table.set_property('column-spacing', 1)
 
-		text = helpers.get_notification_icon_tooltip_text()
-		text = gtkgui_helpers.escape_for_pango_markup(text)
-		
-		self.add_text_row(text)
 		self.hbox.add(self.table)
 		self.win.add(self.hbox)
 
@@ -290,7 +306,7 @@ class GCTooltip(BaseTooltip):
 		properties = []
 
 		nick_markup = '<b>' + \
-			gtkgui_helpers.escape_for_pango_markup(contact.get_shown_name()) \
+			gobject.markup_escape_text(contact.get_shown_name()) \
 			+ '</b>' 
 		properties.append((nick_markup, None))
 
@@ -298,7 +314,7 @@ class GCTooltip(BaseTooltip):
 			status = contact.status.strip()
 			if status != '':
 				# escape markup entities
-				status = helpers.reduce_chars_newlines(status, 100, 5)
+				status = helpers.reduce_chars_newlines(status, 300, 5)
 				status = '<i>' +\
 					gobject.markup_escape_text(status) + '</i>'
 				properties.append((status, None))
@@ -312,7 +328,7 @@ class GCTooltip(BaseTooltip):
 
 		if hasattr(contact, 'resource') and contact.resource.strip() != '':
 			properties.append((_('Resource: '), 
-				gtkgui_helpers.escape_for_pango_markup(contact.resource) ))
+				gobject.markup_escape_text(contact.resource) ))
 		if contact.affiliation != 'none':
 			uf_affiliation = helpers.get_uf_affiliation(contact.affiliation)
 			affiliation_str = \
@@ -323,15 +339,13 @@ class GCTooltip(BaseTooltip):
 		# Add avatar
 		puny_name = helpers.sanitize_filename(contact.name)
 		puny_room = helpers.sanitize_filename(contact.room_jid)
-		for type_ in ('jpeg', 'png'):
-			file = os.path.join(gajim.AVATAR_PATH, puny_room,
-				puny_name + '.' + type_)
-			if os.path.exists(file):
-				self.avatar_image.set_from_file(file)
-				pix = self.avatar_image.get_pixbuf()
-				pix = gtkgui_helpers.get_scaled_pixbuf(pix, 'tooltip')
-				self.avatar_image.set_from_pixbuf(pix)
-				break
+		file = helpers.get_avatar_path(os.path.join(gajim.AVATAR_PATH, puny_room,
+			puny_name))
+		if file:
+			self.avatar_image.set_from_file(file)
+			pix = self.avatar_image.get_pixbuf()
+			pix = gtkgui_helpers.get_scaled_pixbuf(pix, 'tooltip')
+			self.avatar_image.set_from_pixbuf(pix)
 		else:
 			self.avatar_image.set_from_pixbuf(None)
 		while properties:
@@ -355,6 +369,7 @@ class GCTooltip(BaseTooltip):
 					vertical_fill, 0, 0)
 			else:
 				label.set_markup(property[0])
+				label.set_line_wrap(True)
 				vcard_table.attach(label, 1, 3, vcard_current_row,
 					vcard_current_row + 1, gtk.FILL, vertical_fill, 0)
 		
@@ -376,34 +391,31 @@ class RosterTooltip(NotificationAreaTooltip):
 
 	def populate(self, contacts):
 		self.create_window()
-		
+
 		self.create_table()
 		if not contacts or len(contacts) == 0:
 			# Tooltip for merged accounts row
-			accounts = helpers.get_accounts_info()
+			accounts = helpers.get_notification_icon_tooltip_dict()
 			self.table.resize(2, 1)
 			self.spacer_label = ''
 			self.fill_table_with_accounts(accounts)
 			self.win.add(self.table)
 			return
 		
-		
 		# primary contact
 		prim_contact = gajim.contacts.get_highest_prio_contact_from_contacts(
 			contacts)
 		
 		puny_jid = helpers.sanitize_filename(prim_contact.jid)
 		table_size = 3
-		
-		for type_ in ('jpeg', 'png'):
-			file = os.path.join(gajim.AVATAR_PATH, puny_jid + '.' + type_)
-			if os.path.exists(file):
-				self.avatar_image.set_from_file(file)
-				pix = self.avatar_image.get_pixbuf()
-				pix = gtkgui_helpers.get_scaled_pixbuf(pix, 'tooltip')
-				self.avatar_image.set_from_pixbuf(pix)
-				table_size = 4
-				break
+
+		file = helpers.get_avatar_path(os.path.join(gajim.AVATAR_PATH, puny_jid))
+		if file:
+			self.avatar_image.set_from_file(file)
+			pix = self.avatar_image.get_pixbuf()
+			pix = gtkgui_helpers.get_scaled_pixbuf(pix, 'tooltip')
+			self.avatar_image.set_from_pixbuf(pix)
+			table_size = 4
 		else:
 			self.avatar_image.set_from_pixbuf(None)
 		vcard_table = gtk.Table(table_size, 1)
@@ -413,10 +425,17 @@ class RosterTooltip(NotificationAreaTooltip):
 		properties = []
 
 		name_markup = u'<span weight="bold">' + \
-			gtkgui_helpers.escape_for_pango_markup(prim_contact.get_shown_name())\
+			gobject.markup_escape_text(prim_contact.get_shown_name())\
 			+ '</span>'
+		if self.account and prim_contact.jid in gajim.connections[
+		self.account].blocked_contacts:
+			name_markup += _(' [blocked]')
+		if self.account and \
+		self.account in gajim.interface.minimized_controls and \
+		prim_contact.jid in gajim.interface.minimized_controls[self.account]:
+			name_markup += _(' [minimized]')
 		properties.append((name_markup, None))
-		
+
 		num_resources = 0
 		# put contacts in dict, where key is priority
 		contacts_dict = {}
@@ -433,17 +452,15 @@ class RosterTooltip(NotificationAreaTooltip):
 			transport = gajim.get_transport_name_from_jid(
 				prim_contact.jid)
 			if transport:
-				file_path = os.path.join(gajim.DATA_DIR, 'iconsets', 
-					'transports', transport , '16x16')
+				file_path = os.path.join(helpers.get_transport_path(transport),
+					'16x16')
 			else:
 				iconset = gajim.config.get('iconset')
 				if not iconset:
 					iconset = 'dcraven'
-				file_path = os.path.join(gajim.DATA_DIR,
-					'iconsets', iconset, '16x16')
+				file_path = os.path.join(helpers.get_iconset_path(iconset), '16x16')
 
-			contact_keys = contacts_dict.keys()
-			contact_keys.sort()
+			contact_keys = sorted(contacts_dict.keys())
 			contact_keys.reverse()
 			for priority in contact_keys:
 				for contact in contacts_dict[priority]:
@@ -454,6 +471,7 @@ class RosterTooltip(NotificationAreaTooltip):
 					self.add_status_row(file_path, icon_name, status_line,
 						contact.last_status_time)
 			properties.append((self.table,	None))
+
 		else: # only one resource
 			if contact.show:
 				show = helpers.get_uf_show(contact.show) 
@@ -477,6 +495,12 @@ class RosterTooltip(NotificationAreaTooltip):
 						locale.getpreferredencoding())
 					text = text % local_time 
 					show += text
+				if self.account and \
+				prim_contact.jid in gajim.gc_connected[self.account]:
+					if gajim.gc_connected[self.account][prim_contact.jid]:
+						show = _('Connected')
+					else:
+						show = _('Disconnected')
 				show = '<i>' + show + '</i>'
 				# we append show below
 				
@@ -484,25 +508,29 @@ class RosterTooltip(NotificationAreaTooltip):
 					status = contact.status.strip()
 					if status:
 						# reduce long status
-						# (no more than 100 chars on line and no more than 5 lines)
-						status = helpers.reduce_chars_newlines(status, 100, 5)
+						# (no more than 300 chars on line and no more than 5 lines)
+						# status is wrapped
+						status = helpers.reduce_chars_newlines(status, 300, 5)
 						# escape markup entities. 
-						status = gtkgui_helpers.escape_for_pango_markup(status)
+						status = gobject.markup_escape_text(status)
 						properties.append(('<i>%s</i>' % status, None))
 				properties.append((show, None))
+				
+		self._append_pep_info(contact, properties)
 		
 		properties.append((_('Jabber ID: '), prim_contact.jid ))
 
 		# contact has only one ressource
 		if num_resources == 1 and contact.resource:
 			properties.append((_('Resource: '),
-				gtkgui_helpers.escape_for_pango_markup(contact.resource) +\
+				gobject.markup_escape_text(contact.resource) +\
 				' (' + unicode(contact.priority) + ')'))
 		
-		if prim_contact.sub and prim_contact.sub != 'both':
+		if self.account and prim_contact.sub and prim_contact.sub != 'both' and\
+		prim_contact.jid not in gajim.gc_connected[self.account]:
 			# ('both' is the normal sub so we don't show it)
 			properties.append(( _('Subscription: '), 
-				gtkgui_helpers.escape_for_pango_markup(helpers.get_uf_sub(prim_contact.sub))))
+				gobject.markup_escape_text(helpers.get_uf_sub(prim_contact.sub))))
 	
 		if prim_contact.keyID:
 			keyID = None
@@ -512,7 +540,7 @@ class RosterTooltip(NotificationAreaTooltip):
 				keyID = prim_contact.keyID[8:]
 			if keyID:
 				properties.append((_('OpenPGP: '),
-					gtkgui_helpers.escape_for_pango_markup(keyID)))
+					gobject.markup_escape_text(keyID)))
 		
 		while properties:
 			property = properties.pop(0)
@@ -525,7 +553,7 @@ class RosterTooltip(NotificationAreaTooltip):
 			if property[1]:
 				label.set_markup(property[0])
 				vcard_table.attach(label, 1, 2, vcard_current_row,
-					vcard_current_row + 1, gtk.FILL,  vertical_fill, 0, 0)
+					vcard_current_row + 1, gtk.FILL, vertical_fill, 0, 0)
 				label = gtk.Label()
 				label.set_alignment(0, 0)
 				label.set_markup(property[1])
@@ -535,7 +563,8 @@ class RosterTooltip(NotificationAreaTooltip):
 						vertical_fill, 0, 0)
 			else:
 				if isinstance(property[0], (unicode, str)): #FIXME: rm unicode?
-					label.set_markup(property[0])
+					label.set_markup(property[0]) 
+					label.set_line_wrap(True)
 				else:
 					label = property[0]
 				vcard_table.attach(label, 1, 3, vcard_current_row,
@@ -546,6 +575,73 @@ class RosterTooltip(NotificationAreaTooltip):
 				vcard_current_row + 1, gtk.FILL, gtk.FILL | gtk.EXPAND, 3, 3)
 		self.win.add(vcard_table)
 
+	def _append_pep_info(self, contact, properties):
+		'''
+		Append Tune, Mood, Activity information of the specified contact
+		to the given property list.
+		'''
+		if 'mood' in contact.mood:
+			mood = contact.mood['mood'].strip()
+			mood = MOODS.get(mood, mood)
+			mood = gobject.markup_escape_text(mood)
+			mood_string = _('Mood:') + ' <b>%s</b>' % mood
+			if 'text' in contact.mood \
+			and contact.mood['text'] != '':
+				mood_text = contact.mood['text'].strip()
+				mood_text = \
+					gobject.markup_escape_text(mood_text)
+				mood_string += ' (%s)' % mood_text
+			properties.append((mood_string, None))
+
+		if 'activity' in contact.activity:
+			activity = act_plain = \
+				contact.activity['activity'].strip()
+			activity = gobject.markup_escape_text(activity)
+			if act_plain in ACTIVITIES:
+				activity = ACTIVITIES[activity]['category']
+			activity_string = _('Activity:') + ' <b>%s' % activity
+			if 'subactivity' in contact.activity:
+				activity_sub = \
+					contact.activity['subactivity'].strip()
+				if act_plain in ACTIVITIES and activity_sub in \
+				ACTIVITIES[act_plain]:
+					activity_sub = ACTIVITIES[act_plain][activity_sub]
+				activity_sub = \
+					gobject.markup_escape_text(activity_sub)
+				activity_string += ': %s</b>' % activity_sub
+			else:
+				activity_string += '</b>'
+			if 'text' in contact.activity:
+				activity_text = contact.activity['text'].strip()
+				activity_text = gobject.markup_escape_text(
+					activity_text)
+				activity_string += ' (%s)' % activity_text
+			properties.append((activity_string, None))
+
+		if 'artist' in contact.tune \
+		or 'title' in contact.tune:
+			if 'artist' in contact.tune:
+				artist = contact.tune['artist'].strip()
+				artist = gobject.markup_escape_text(artist)
+			else:
+				artist = _('Unknown Artist')
+			if 'title' in contact.tune:
+				title = contact.tune['title'].strip()
+				title = gobject.markup_escape_text(title)
+			else:
+				title = _('Unknown Title')
+			if 'source' in contact.tune:
+				source = contact.tune['source'].strip()
+				source = gobject.markup_escape_text(source)
+			else:
+				source = _('Unknown Source')
+			tune_string = _('Tune:') + ' ' + \
+				_('<b>"%(title)s"</b> by <i>%(artist)s</i>\n'
+				'from <i>%(source)s</i>') % {'title': title,
+				'artist': artist, 'source': source}
+			properties.append((tune_string, None))
+			
+
 class FileTransfersTooltip(BaseTooltip):
 	''' Tooltip that is shown in the notification area '''
 	def __init__(self):
@@ -563,9 +659,9 @@ class FileTransfersTooltip(BaseTooltip):
 		else:
 			file_name = file_props['name']
 		properties.append((_('Name: '), 
-			gtkgui_helpers.escape_for_pango_markup(file_name)))
+			gobject.markup_escape_text(file_name)))
 		if file_props['type'] == 'r':
-			type =  _('Download')
+			type = _('Download')
 			actor = _('Sender: ') 
 			sender = unicode(file_props['sender']).split('/')[0]
 			name = gajim.contacts.get_first_contact_from_jid( 
@@ -579,37 +675,39 @@ class FileTransfersTooltip(BaseTooltip):
 			else:
 				name = receiver.split('/')[0]
 		properties.append((_('Type: '), type))
-		properties.append((actor, gtkgui_helpers.escape_for_pango_markup(name)))
+		properties.append((actor, gobject.markup_escape_text(name)))
 		
-		transfered_len = 0
-		if file_props.has_key('received-len'):
-			transfered_len = file_props['received-len']
+		transfered_len = file_props.get('received-len', 0)
 		properties.append((_('Transferred: '), helpers.convert_bytes(transfered_len)))
 		status = '' 
-		if not file_props.has_key('started') or not file_props['started']:
-			status =  _('Not started')
-		elif file_props.has_key('connected'):
-			if file_props.has_key('stopped') and \
+		if 'started' not in file_props or not file_props['started']:
+			status = _('Not started')
+		elif 'connected' in file_props:
+			if 'stopped' in file_props and \
 			file_props['stopped'] == True:
 				status = _('Stopped')
 			elif file_props['completed']:
-					status = _('Completed')
+				status = _('Completed')
 			elif file_props['connected'] == False:
 				if file_props['completed']:
 					status = _('Completed')
 			else:
-				if file_props.has_key('paused') and  \
+				if 'paused' in file_props and \
 				file_props['paused'] == True:
 					status = _('?transfer status:Paused')
-				elif file_props.has_key('stalled') and \
+				elif 'stalled' in file_props and \
 				file_props['stalled'] == True:
 					#stalled is not paused. it is like 'frozen' it stopped alone
 					status = _('Stalled')
 				else:
 					status = _('Transferring')
 		else:
-			status =  _('Not started')
+			status = _('Not started')
 		properties.append((_('Status: '), status))
+		if 'desc' in file_props:
+			file_desc = file_props['desc']
+			properties.append((_('Description: '), gobject.markup_escape_text(
+				file_desc)))
 		while properties:
 			property = properties.pop(0)
 			current_row += 1
@@ -617,7 +715,7 @@ class FileTransfersTooltip(BaseTooltip):
 			label.set_alignment(0, 0)
 			label.set_markup(property[0])
 			ft_table.attach(label, 1, 2, current_row, current_row + 1, 
-				gtk.FILL,  gtk.FILL, 0, 0)
+				gtk.FILL, gtk.FILL, 0, 0)
 			label = gtk.Label()
 			label.set_alignment(0, 0)
 			label.set_line_wrap(True)
@@ -644,3 +742,5 @@ class ServiceDiscoveryTooltip(BaseTooltip):
 				_('This service could not respond with detailed information.\n'
 				'It is most likely legacy or broken'))
 		self.win.add(label)
+
+# vim: se ts=3:
